@import "@patternfly/patternfly/base/patternfly-variables.scss";

/* Bootstrap scss compilation will complain if these are not set
 * https://github.com/twbs/bootstrap/issues/23982
 */
$grid-gutter-width: 30px;
$font-size-base: 1;

/*
 * PatternFly 4 adapting the lists too early.
 * When PF4 has a breakpoint of 768px width, it's actually 1108 for us, as the sidebar is 340px.
 * (This does use the intended content area, but there's a mismatch between content and browser width as we use iframes.)
 * So redefine grid breakpoints
 */
$pf-global--breakpoint--xs: 0 !default;
$pf-global--breakpoint--sm: 236px !default;
$pf-global--breakpoint--md: 428px !default;
$pf-global--breakpoint--lg: 652px !default;
$pf-global--breakpoint--xl: 860px !default;
$pf-global--breakpoint--2xl: 1110px !default;
